/*
 * World Storage API
 *
 * API ensuring interoperability between an authoring tool and a World Storage service
 *
 * The version of the OpenAPI document: 0.0.4
 * 
 * Generated by: https://openapi-generator.tech
 */

using System;
using System.Collections.Generic;
using System.ComponentModel.DataAnnotations;
using Microsoft.AspNetCore.Authorization;
using Microsoft.AspNetCore.Mvc;
using Microsoft.AspNetCore.Http;
using Swashbuckle.AspNetCore.Annotations;
using Swashbuckle.AspNetCore.SwaggerGen;
using Newtonsoft.Json;
using Org.OpenAPITools.Attributes;
using Org.OpenAPITools.Models;
using Org.OpenAPITools.Services;
using MongoDB.Driver;

namespace Org.OpenAPITools.Controllers
{
  /// <summary>
  /// 
  /// </summary>
  [ApiController]
  public class WorldAnchorsApiControllerImpl : WorldAnchorsApiController
  {

    private readonly WorldAnchorService _worldAnchorService;

    /// <summary>
    /// 
    /// </summary>
    public WorldAnchorsApiControllerImpl(WorldAnchorService worldAnchorService)
    {
      _worldAnchorService = worldAnchorService;
    }


    /// <summary>
    /// Create a world anchor.
    /// </summary>
    /// <param name="worldAnchor">The world anchor to be added to the world storage.</param>
    /// <response code="200">OK, returns the UUID of the World Anchor defined by the world storage.</response>
    /// <response code="201">Null response.</response>
    /// <response code="400">Bad request.</response>
    /// <response code="409">Invalid UUID, id must be a Nil value.</response>
    /// <response code="0">Unexpected error.</response>
    [HttpPost]
    [Route("/worldAnchors")]
    [Consumes("application/json")]
    [ValidateModelState]
    [SwaggerOperation("AddWorldAnchor")]
    [SwaggerResponse(statusCode: 200, type: typeof(string), description: "OK, returns the UUID of the World Anchor defined by the world storage.")]
    [SwaggerResponse(statusCode: 0, type: typeof(Error), description: "Unexpected error.")]
    public override IActionResult AddWorldAnchor([FromBody] WorldAnchor worldAnchor)
    {
      if (String.IsNullOrEmpty(worldAnchor.UUID.ToString()))
      {
        worldAnchor.UUID = Guid.NewGuid();
      }
      try
      {
        WorldAnchor myworldanchor = _worldAnchorService.Create(worldAnchor);
        return new ObjectResult(myworldanchor);
      }
      catch (Exception e)
      {
        return StatusCode(400, e.Message);
      }
    }

    /// <summary>
    /// Deletes a world anchor.
    /// </summary>
    /// <param name="worldAnchorUUID">World anchor UUID to delete.</param>
    /// <response code="200">OK, delete successful.</response>
    /// <response code="400">Invalid UUID supplied.</response>
    /// <response code="404">Not found, could not find UUID in database.</response>
    [HttpDelete]
    [Route("/worldAnchors/{worldAnchorUUID}")]
    [ValidateModelState]
    [SwaggerOperation("DeleteWorldAnchor")]
    public override IActionResult DeleteWorldAnchor([FromRoute(Name = "worldAnchorUUID")][Required] Guid worldAnchorUUID)
    {
      DeleteResult answer = _worldAnchorService.Remove((worldAnchorUUID));
      // check, if used in WorldLink
      string result = "ok";
      string worldlinkinfo = "";
      List<WorldLink> worldlinklistfrom = _worldAnchorService.GetWorldLinkUUIDFrom(worldAnchorUUID);
      List<WorldLink> worldlinklistto = _worldAnchorService.GetWorldLinkUUIDTo(worldAnchorUUID);
      foreach (WorldLink worldlink in worldlinklistfrom)
      {
        worldlinkinfo += worldlink.UUID.ToString() + "; ";
      }
      foreach (WorldLink worldlink in worldlinklistto)
      {
        worldlinkinfo += worldlink.UUID.ToString() + "; ";
      }
      if (worldlinkinfo.Length > 1)
      {
        result += ", but removed object is still referenced in " + worldlinkinfo;
      }
      return (answer.IsAcknowledged && answer.DeletedCount > 0) ? new ObjectResult(result) : StatusCode(404, "Not found, could not find UUID in database.");      
    }

    /// <summary>
    /// Find a world anchor by its UUID.
    /// </summary>
    /// <param name="worldAnchorUUID">UUID of the world anchor to retrieve.</param>
    /// <response code="200">Successful operation.</response>
    /// <response code="400">Invalid UUID supplied.</response>
    /// <response code="404">Not found, could not find UUID in database.</response>
    [HttpGet]
    [Route("/worldAnchors/{worldAnchorUUID}")]
    [ValidateModelState]
    [SwaggerOperation("GetWorldAnchorById")]
    [SwaggerResponse(statusCode: 200, type: typeof(WorldAnchor), description: "Successful operation.")]
    public override IActionResult GetWorldAnchorById([FromRoute(Name = "worldAnchorUUID")][Required] Guid worldAnchorUUID)
    {
      WorldAnchor myworldanchor = _worldAnchorService.Get(worldAnchorUUID);
      return (null != myworldanchor) ? new ObjectResult(myworldanchor) : StatusCode(404, "Not found, could not find UUID in database.");
    }

    /// <summary>
    /// Returns the list of all world anchors defined by the world storage.
    /// </summary>
    /// <response code="200">OK, returns all the world anchors defined by the world storage.</response>
    /// <response code="201">Null response.</response>
    /// <response code="0">Unexpected error.</response>
    [HttpGet]
    [Route("/worldAnchors")]
    [ValidateModelState]
    [SwaggerOperation("GetWorldAnchors")]
    [SwaggerResponse(statusCode: 200, type: typeof(List<WorldAnchor>), description: "OK, returns all the world anchors defined by the world storage.")]
    [SwaggerResponse(statusCode: 0, type: typeof(Error), description: "Unexpected error.")]
    public override IActionResult GetWorldAnchors()
    {
      List<WorldAnchor> worldanchorlist = _worldAnchorService.Get();
      return new ObjectResult(worldanchorlist);
    }
  }
}
