﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;
using Org.OpenAPITools.Models;
using MongoDB.Driver;

namespace Org.OpenAPITools.Services
{
  public class WorldLinkService
  {
    private readonly IMongoCollection<WorldLink> _worldlinkcollection;
    private readonly IMongoCollection<WorldAnchor> _worldanchorcollection;
    private readonly IMongoCollection<Trackable> _trackablecollection;

    /// <summary>
    /// 
    /// </summary>
    public WorldLinkService(IDatabaseSettings settings)
    {
      var client = new MongoClient(settings.ConnectionString);
      var database = client.GetDatabase(settings.DatabaseName);

      _worldlinkcollection = database.GetCollection<WorldLink>(settings.CollectionNameWorldLink);
      _worldanchorcollection = database.GetCollection<WorldAnchor>(settings.CollectionNameWorldAnchor);
      _trackablecollection = database.GetCollection<Trackable>(settings.CollectionNameTrackables);
    }

    public List<WorldLink> Get() =>
        _worldlinkcollection.Find(worldlink => true).ToList();

    public List<WorldLink> Get(int limit) =>
        _worldlinkcollection.Find(worldlink => true).Limit(limit).ToList();

    public WorldLink Get(Guid UUID) =>
        _worldlinkcollection.Find<WorldLink>(worldlink => worldlink.UUID == UUID).FirstOrDefault();

    public WorldLink Create(WorldLink worldlink)
    {
      _worldlinkcollection.InsertOne(worldlink);
      return worldlink;
    }

    public ReplaceOneResult Update(Guid UUID, WorldLink worldlinkIn) =>
        _worldlinkcollection.ReplaceOne(worldlink => worldlink.UUID == UUID, worldlinkIn);

    public DeleteResult Remove(WorldLink worldlinkIn) =>
        _worldlinkcollection.DeleteOne(worldlink => worldlink.UUID == worldlinkIn.UUID);

    public DeleteResult Remove(Guid UUID) =>
        _worldlinkcollection.DeleteOne(worldlink => worldlink.UUID == UUID);


    /*********************/
    public WorldAnchor GetAnchor(Guid UUID) =>
        _worldanchorcollection.Find<WorldAnchor>(worldanchor => worldanchor.UUID == UUID).FirstOrDefault();

    public Trackable GetTrackable(Guid UUID) =>
        _trackablecollection.Find<Trackable>(trackable => trackable.UUID == UUID).FirstOrDefault();

  }
}

