/*
 * World Storage API
 *
 * API ensuring interoperability between an authoring tool and a World Storage service
 *
 * The version of the OpenAPI document: 0.0.4
 * 
 * Generated by: https://openapi-generator.tech
 */

using System;
using System.IO;
using System.Reflection;
using Microsoft.AspNetCore.Authorization;
using Microsoft.AspNetCore.Builder;
using Microsoft.AspNetCore.Hosting;
using Microsoft.Extensions.Configuration;
using Microsoft.Extensions.DependencyInjection;
using Microsoft.Extensions.Hosting;
using Microsoft.OpenApi.Models;
using Newtonsoft.Json.Converters;
using Newtonsoft.Json.Serialization;
using Org.OpenAPITools.Authentication;
using Org.OpenAPITools.Filters;
using Org.OpenAPITools.OpenApi;
using Org.OpenAPITools.Formatters;
using Org.OpenAPITools.Services;
using MongoDB.Bson.Serialization;
using MongoDB.Bson;
using MongoDB.Bson.Serialization.Serializers;
using Microsoft.Extensions.Options;

namespace Org.OpenAPITools
{
  /// <summary>
  /// Startup
  /// </summary>
  public class Startup
  {
    /// <summary>
    /// Constructor
    /// </summary>
    /// <param name="configuration"></param>
    public Startup(IConfiguration configuration)
    {
      Configuration = configuration;
    }

    /// <summary>
    /// The application configuration.
    /// </summary>
    public IConfiguration Configuration { get; }

    /// <summary>
    /// This method gets called by the runtime. Use this method to add services to the container.
    /// </summary>
    /// <param name="services"></param>
    public void ConfigureServices(IServiceCollection services)
    {
      BsonSerializer.RegisterSerializer(new GuidSerializer(GuidRepresentation.Standard));
      BsonDefaults.GuidRepresentationMode = GuidRepresentationMode.V3;
      // requires using Microsoft.Extensions.Options
      services.Configure<DatabaseSettings>(
            Configuration.GetSection(nameof(DatabaseSettings)));
      services.AddSingleton<IDatabaseSettings>(sp =>
             sp.GetRequiredService<IOptions<DatabaseSettings>>().Value);
      services.AddSingleton<TrackableService>();
      services.AddSingleton<WorldAnchorService>();
      services.AddSingleton<WorldLinkService>();

      // Add framework services.
      services
          // Don't need the full MVC stack for an API, see https://andrewlock.net/comparing-startup-between-the-asp-net-core-3-templates/
          .AddControllers(options =>
          {
            options.InputFormatters.Insert(0, new InputFormatterStream());
          })
          .AddNewtonsoftJson(opts =>
          {
                  opts.SerializerSettings.ContractResolver = new DefaultContractResolver();
/*                  opts.SerializerSettings.ContractResolver = new CamelCasePropertyNamesContractResolver();
            opts.SerializerSettings.Converters.Add(new StringEnumConverter
            {
              NamingStrategy = new CamelCaseNamingStrategy()
                  });*/
          });

      services
          .AddSwaggerGen(c =>
          {
            c.SwaggerDoc("0.0.4", new OpenApiInfo
            {
              Title = "World Storage API",
              Description = "World Storage API (ASP.NET Core 3.1)",
              TermsOfService = new Uri("https://github.com/openapitools/openapi-generator"),
              Contact = new OpenApiContact
              {
                Name = "OpenAPI-Generator Contributors",
                Url = new Uri("https://github.com/openapitools/openapi-generator"),
                Email = ""
              },
              License = new OpenApiLicense
              {
                Name = "NoLicense",
                Url = new Uri("https://opensource.org/licenses/BSD-3-Clause")
              },
              Version = "0.0.4",
            });
            c.CustomSchemaIds(type => type.FriendlyId(true));
            c.IncludeXmlComments($"{AppContext.BaseDirectory}{Path.DirectorySeparatorChar}{Assembly.GetEntryAssembly().GetName().Name}.xml");

                  // Include DataAnnotation attributes on Controller Action parameters as OpenAPI validation rules (e.g required, pattern, ..)
                  // Use [ValidateModelState] on Actions to actually validate it in C# as well!
                  c.OperationFilter<GeneratePathParamsValidationFilter>();
          });
      services
          .AddSwaggerGenNewtonsoftSupport();
    }

    /// <summary>
    /// This method gets called by the runtime. Use this method to configure the HTTP request pipeline.
    /// </summary>
    /// <param name="app"></param>
    /// <param name="env"></param>
    public void Configure(IApplicationBuilder app, IWebHostEnvironment env)
    {
      if (env.IsDevelopment())
      {
        app.UseDeveloperExceptionPage();
      }
      else
      {
        app.UseHsts();
      }

      app.UseHttpsRedirection();
      app.UseDefaultFiles();
      app.UseStaticFiles();
      app.UseSwagger(c =>
          {
            c.RouteTemplate = "openapi/{documentName}/openapi.json";
          })
          .UseSwaggerUI(c =>
          {
        // set route prefix to openapi, e.g. http://localhost:8080/openapi/index.html
        c.RoutePrefix = "openapi";
        //TODO: Either use the SwaggerGen generated OpenAPI contract (generated from C# classes)
            c.SwaggerEndpoint("/openapi/0.0.4/openapi.json", "World Storage API");

        //TODO: Or alternatively use the original OpenAPI contract that's included in the static files
        // c.SwaggerEndpoint("/openapi-original.json", "World Storage API Original");
      });
      app.UseRouting();
      app.UseEndpoints(endpoints =>
          {
            endpoints.MapControllers();
          });
    }
  }
}
